import { applyMiddleware, createStore, compose } from "redux";
import * as Sentry from "@sentry/react";
import logger from "redux-logger";
import createSagaMiddleware from "redux-saga";
import thunk from "redux-thunk";
import rootReducer from "./reducers";
import rootSaga from "./sagas";
import { showNotification } from "../config/notifications";

const bindMiddleware = (middleware) => applyMiddleware(...middleware);

function configureStore(initialState = {}) {
  const sagaMiddleware = createSagaMiddleware({
    onError: (error) => {
      showNotification({ message: error.name, description: error.message, type: "error" });
    },
  });
  const middlewares = [thunk, sagaMiddleware];
  if (process.env.NODE_ENV === "development") {
    middlewares.push(logger);
  }
  const store = createStore(
    rootReducer,
    initialState,
    compose(
      bindMiddleware(middlewares),
      process.env.REACT_APP_SENTRY_REDUX_LOG === "true" ? Sentry.createReduxEnhancer() : null,
    ),
  );

  store.sagaTask = sagaMiddleware.run(rootSaga);
  return store;
}

const store = configureStore();
export default store;
